/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.db.Abstract');

goog.require('goog.Disposable');
goog.require('goog.structs.Map');

goog.require('pricecheck.json.json');



/**
 * An abstract localStorage store. It allows you to create simple key => value
 * databases which are persisted to localStorage where possible.
 * You must set this.namespace_ when extending this class.
 * @param {string} namespace The namespace of the database.
 * @param {Storage=} opt_storage The HTML5 localStorage object.
 * @param {goog.structs.Map=} opt_map An optional map to use as an in memory
 *      database.
 * @extends {goog.Disposable}
 * @constructor
 */
pricecheck.db.Abstract = function(namespace, opt_storage, opt_map) {
    goog.Disposable.call(this);

    /**
     * The namespace of the database.
     * @type {string}
     * @private
     */
    this.namespace_ = namespace;
    if (!goog.isString(this.namespace_)) {
        throw 'No namespace set.';
    }

    /**
     * A reference to window.localStorage.
     * @type {Storage}
     * @private
     */
    this.storage_ = opt_storage || window['localStorage'];

    /**
     * The in-memory collection of items.
     * @type {goog.structs.Map}
     * @private
     */
    this.map_ = opt_map || new goog.structs.Map();
    if (pricecheck.device.getSupport().LOCAL_STORAGE) {
        this.map_.addAll(
            pricecheck.json.json.parse(this.storage_.getItem(this.namespace_)));
    }
};
goog.inherits(pricecheck.db.Abstract, goog.Disposable);


/**
 * Returns the namespace of the database.
 * @return {string} The database namespace.
 */
pricecheck.db.Abstract.prototype.getNamespace = function() {
    return this.namespace_;
};


/**
 * Whether or not the client has localStorage available.
 * @return {boolean} Whether the client has localStorage.
 */
pricecheck.db.Abstract.prototype.hasLocalStorage = function() {
    return pricecheck.device.getSupport().LOCAL_STORAGE;
};


/**
 * Return the in memory collection map.
 * @return {goog.structs.Map} The map.
 */
pricecheck.db.Abstract.prototype.getMap = function() {
    if (!this.map_) {
        this.map_ = new goog.structs.Map();
    }
    return this.map_;
};



/**
 * Returns all items in the collection.
 * @return {Object} An object containing all items in collection.
 */
pricecheck.db.Abstract.prototype.findAll = function() {
    return this.getMap().toObject();
};


/**
 * Return the item for the given key.
 * @param {string|number} id The key of the item to return.
 * @return {*} The value of the specified key.
 */
pricecheck.db.Abstract.prototype.findById = function(id) {
    return this.getMap().get(id);
};


/**
 * Determine whether an item is set or not.
 * @param {string|number} id The key of the item to check for.
 * @return {boolean} Whether a key exists or not.
 */
pricecheck.db.Abstract.prototype.exists = function(id) {
    return this.getMap().containsKey(id);
};


/**
 * Save an item to localStorage.
 * @param {string|number} id The key to store the item under.
 * @param {*} item The item to store.
 */
pricecheck.db.Abstract.prototype.set = function(id, item) {
    this.getMap().set(id, item);
    this.save();
};


/**
 * Remove an item from the collection.
 * @param {string|number} id The key of the item to remove.
 */
pricecheck.db.Abstract.prototype.remove = function(id) {
    this.getMap().remove(id);
    this.save();
};


/**
 * Remove all items from the collection.
 */
pricecheck.db.Abstract.prototype.clear = function() {
    this.getMap().clear();
    this.storage_.clear();
};


/**
 * Persist the in-memory collection to localStorage.
 */
pricecheck.db.Abstract.prototype.save = function() {
    if (!pricecheck.device.getSupport().LOCAL_STORAGE) {
        return;
    }
    try {
        this.storage_.setItem(this.namespace_,
            pricecheck.json.json.stringify(this.getMap().toObject()));
    } catch (e) {
        // Probably browsing in "Private mode" or quota is exceeded.
        // TODO: handle accordingly.
    }
};


/**
 * @inheritDoc
 */
pricecheck.db.Abstract.prototype.disposeInternal = function() {
    goog.base(this, 'disposeInternal');

};
