goog.provide('pricecheck.Device');

goog.require('goog.style.transition');



/**
 * A class for determining the mobile device OS and capabilities.
 *
 * @constructor
 */
pricecheck.Device = function() {

    /**
     * The browser user agent.
     * @type {?string}
     * @private
     */
    this.userAgent_ = navigator.userAgent;

    /**
     * The device OS.
     * @type {?string}
     * @private
     */
    this.device_ = null;

    /**
     * The device OS version.
     * @type {?number}
     * @private
     */
    this.version_ = null;

    /**
     * The device Appstore URL.
     * @type {?string}
     * @private
     */
    this.appUrl_ = null;

    /**
     * An enum-like object containing device browser capabilities.
     * @type {Object.<boolean>}
     * @private
     */
    this.support_ = {
        TOUCH_EVENTS: false,
        CSS3_TRANSITIONS: false,
        LOCAL_STORAGE: false
    };

    this.init_();
};


/**
 * Initialize the device OS and version by examining the user agent string.
 * @private
 */
pricecheck.Device.prototype.init_ = function() {

    // Detect OS and version
    if ((/android/i).test(this.userAgent_)) {
        this.device_ = pricecheck.Device.os.ANDROID;
        this.appUrl_ =
            'https://play.google.com/store/apps/details?id=com.pricecheck';
        /*
        // If Samsung device direct to Samung store - no idea why.
        if (pricecheck.device == 'android' &&
            (/GT-/).test(goog.userAgent.getUserAgentString())) {
            appLink = pricecheck.ui.page.barcode.Barcode.AppUrl['samsung'];
        }
        'http://www.samsungapps.com/topApps/topAppsDetail.as?productId=G00008523394'
         */
    } else if ((/ipod|iphone/i).test(this.userAgent_)) {
        this.device_ = pricecheck.Device.os.IOS;
        this.appUrl_ =
            'http://itunes.apple.com/us/app/pricecheck/id539418026?mt=8';
    } else if ((/blackberry/i).test(this.userAgent_)) {
        this.device_ = pricecheck.Device.os.BLACKBERRY;
        this.appUrl_ =
            'http://appworld.blackberry.com/webstore/vendor/14930/?lang=en';
        // Check version
        if (this.userAgent_.indexOf('Version/') >= 0) { //
            var position = this.userAgent_.indexOf('Version/') + 8;
            this.version_ = parseFloat(
                this.userAgent_.substring(position, position + 3));
        }
    } else if ((/iemobile/i).test(this.userAgent_)) {
        this.device_ = pricecheck.Device.os.WP7;
        this.appUrl_ =
            'http://www.windowsphone.com/en-US/apps/0999bc5b-6e81-4b15-93c6-40c837cb32fe';
    }

    // Detect device capabilities
    if ('ontouchstart' in document.documentElement) {
        this.support_.TOUCH_EVENTS = true;
    }
    if (goog.style.transition.isSupported()) {
        this.support_.CSS3_TRANSITIONS = true;
    }
    if (typeof(goog.global['localStorage']) != 'undefined') {
        this.support_.LOCAL_STORAGE = true;
    }
};


/**
 * Return the device OS as one of the below constants.
 * @return {?string} The device OS.
 */
pricecheck.Device.prototype.getOs = function() {
    return this.device_;
};


/**
 * Return the device browser capabilities.
 * @return {Object.<boolean>} The device browser capabilites.
 */
pricecheck.Device.prototype.getSupport = function() {
    return this.support_;
};


/**
 * Return the Appstore url for the device.
 * @return {?string} The Appstore url.
 */
pricecheck.Device.prototype.getAppUrl = function() {
    return this.appUrl_;
};



/**
 * Detect whether the device is using the application or the touch website.
 * @return {boolean} Whether the application is native or not.
 */
pricecheck.Device.prototype.isNative = function() {
    return (typeof(window['cordova']) != 'undefined');
};


/**
 * Check whether the device is Blackberry 6 or not.
 * @return {boolean} Whether the device is Blackberry 6 or not.
 */
pricecheck.Device.prototype.isBB6 = function() {
    if ((this.getOs() == pricecheck.Device.os.BLACKBERRY) &&
        this.version_ == 6) {
        return true;
    } else {
        return false;
    }
};


/**
 * An enum of device OS's.
 * @enum {string}
 */
pricecheck.Device.os = {
    ANDROID: 'android',
    IOS: 'ios',
    BLACKBERRY: 'bb',
    WP7: 'wp7',
    SAMSUNG: 'samsung'
};


