/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.net.Xhr');

goog.require('goog.Uri');
goog.require('goog.net.XhrIo');
goog.require('goog.string');



/**
 * Static method for performing an XHR request.
 * @param {string} url The URL of the request.
 * @param {?Object=} opt_serializedArgs Optional paramaters to be passed in the
 *     request.
 * @param {?function(Object)=} opt_callback Optional success callback.
 * @param {?function(Object)=} opt_errorCallback Optional error callback.
 * @param {?string=} opt_method Optional request method. Default to GET.
 * @param {?boolean=} opt_showFeedback Whether or not to display the feedback
 *     element during the request.
 * @param {?boolean=} opt_json Whether the reponse is JSON or HTML. Defaults to
 *     true.
 */
pricecheck.net.Xhr.send = function(url, opt_serializedArgs, opt_callback, 
    opt_errorCallback, opt_method, opt_showFeedback, opt_json) {
	
    if(!pricecheck.isOnLine()){
		
        var eventBus = goog.pubsub.PubSub.getInstance();
        eventBus.publish('/offline');
        // window.location.href = '#offline';
        return;
    }

    // Set default method
    opt_method = opt_method || 'POST';

    // Set default failure and success callbacks
    opt_errorCallback = opt_errorCallback || function(response) {
        alert(response);
    }
    opt_callback = opt_callback || function() {}

    // Set default show feedback
    opt_showFeedback = opt_showFeedback || false;

    // Show the feedback element if enabled
    if (opt_showFeedback) {
        var feedback = pricecheck.ui.progress.Progress.getInstance();
        if (feedback)
            feedback.setVisible(true);
    }

    // Set default return type
    opt_json = (typeof(opt_json) != 'undefined') ? opt_json : true;

    // Set default arguments
    opt_serializedArgs = opt_serializedArgs || {};

    goog.net.XhrIo.send(url, function(e){
		
        // Hide feedback
        if(feedback)
            feedback.setVisible(false);
        // If request failed, display error
        if(!e.target.isSuccess()){
			
            return;
        }
        // Handle response and invoke callbacks
        var response;
		
        if(opt_json){
			
            response = e.target.getResponseJson();
        }
		else{
			
            response = e.target.getResponse();
        }
        opt_callback(response);
    },
    opt_method,
    pricecheck.net.Xhr.encodeKeyValuePairs(opt_serializedArgs),
    {'X-REQUESTED-WITH' : 'XMLHttpRequest'});
};


/**
 * Return an encoded queryData string from a map or object like variable
 * @param {Object|string} serializedArgs The data to be serialized.
 * @return {?string} The serialized arguments.
 */
pricecheck.net.Xhr.encodeKeyValuePairs = function(serializedArgs) {
    if (!goog.isObject(serializedArgs)) {
        return serializedArgs;
    }
    var queryData = new goog.Uri.QueryData();
    for (var param in serializedArgs)
        queryData.set(param, serializedArgs[param]);
    return queryData.toString();
};
