/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.scroller.Scroller');
goog.provide('pricecheck.scroller.Scroller.Event');

goog.require('goog.events');
goog.require('goog.events.EventTarget');



/**
 * A window scroller helper. Distpatches scroll and scroll-stop events.
 * @param {goog.events.EventHandler=} opt_eh An optional event handler.
 * @constructor
 * @extends {goog.events.EventTarget}
 */
pricecheck.scroller.Scroller = function(opt_eh) {
    goog.events.EventTarget.call(this);

    /**
     * The window object.
     * @type {Window}
     * @private
     */
    this.window_ = window;

    /**
     * The event handler.
     * @type {goog.events.EventHandler}
     * @private
     */
    this.eh_ = opt_eh || new goog.events.EventHandler(this);

    /**
     * Whether the window is currently being scrolled or not.
     * @type {boolean}
     * @private
     */
    this.scrolling_ = false;

    /**
     * The ID of the current timer.
     * @type {?number}
     * @private
     */
    this.timer_ = null;

    /**
     * The latency allowed between scroll events before dispatching scroll-stop.
     * @type {number}
     * @private
     */
    this.latency_ = 100;

};
goog.inherits(pricecheck.scroller.Scroller, goog.events.EventTarget);


/**
 * Start listening for window scroll events.
 */
pricecheck.scroller.Scroller.prototype.start = function() {
    this.eh_.listen(this.window_, goog.events.EventType.SCROLL,
        this.onScroll_);
};


/**
 * Stop listening for window scroll events.
 */
pricecheck.scroller.Scroller.prototype.stop = function() {
    this.eh_.unlisten(this.window_, goog.events.EventType.SCROLL);
};


/**
 * @inheritDoc
 */
pricecheck.scroller.Scroller.prototype.disposeInternal = function() {
    clearTimeout(this.timer_);
    this.eh_.dispose();
};


/**
 * Handle the window scroll event. Dispatches either a scroll or scroll-stop
 * event.
 * @private
 */
pricecheck.scroller.Scroller.prototype.onScroll_ = function() {
    if (!this.scrolling_) {
        this.scrolling_ = true;
        this.dispatchEvent(pricecheck.scroller.Scroller.Event.SCROLL);
    }

    clearTimeout(this.timer_);
    this.timer_ = goog.global.setTimeout(goog.bind(function() {
        this.scrolling_ = false;
        this.dispatchEvent(pricecheck.scroller.Scroller.Event.STOP);
    }, this), this.latency_);
};


/**
 * An enum of scroll events dispatched by this class.
 * @enum {string}
 */
pricecheck.scroller.Scroller.Event = {
    SCROLL: 'scroll',
    STOP: 'scrollstop'
};
