/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.ui.carousel.Carousel');

goog.require('goog.array');
goog.require('goog.async.Delay');
goog.require('goog.dom.classes');
goog.require('goog.dom.dataset');
goog.require('goog.fx.Dragger');
goog.require('goog.fx.dom.SlideFrom');
goog.require('goog.fx.easing');
goog.require('goog.structs');
goog.require('goog.style.transition');
goog.require('goog.ui.Component');

goog.require('pricecheck.ui.FastButton');
goog.require('pricecheck.ui.carousel.template');



/**
 * A carousel component allowing you to cycle through 3 items at a time using
 * a swipe gesture or click. Each item in the list must be the same width.
 * @param {Array.<Object>} items An array of items to render.
 * @param {string} opt_moreLink An optional link to display when the end of the
 *     carousel is reached.
 * @param {goog.dom.DomHelper=} opt_domHelper The DOM helper object for the
 *     document we want to render in.
 * @constructor
 * @extends {goog.ui.Component}
 */
pricecheck.ui.carousel.Carousel = function(items, opt_moreLink, opt_domHelper) {
    goog.ui.Component.call(this, opt_domHelper);

    /**
     * An array of items to render.
     * @type {Array.<Object>}
     * @private
     */
    this.items_ = items;

    /**
     * A link to display when the end of the carousel is reached.
     * @type {?string}
     * @private
     */
    this.moreLink_ = opt_moreLink || null;

    /**
     * The product descrption name element.
     * @type {?Element}
     * @private
     */
    this.nameEl_ = null;

    /**
     * The product descrption price element.
     * @type {?Element}
     * @private
     */
    this.priceEl_ = null;

    /**
     * The carousel stage element.
     * @type {?Element}
     * @private
     */
    this.stageEl_ = null;

    /**
     * The array of carousel elements.
     * @type { {length: number} }
     * @private
     */
    this.itemEls_ = [];

    /**
     * With width of each item in the carousel.
     * @type {?number}
     * @private
     */
    this.itemWidth_ = null;

    /**
     * The current left x co-ordinate of the stage.
     * @type {number}
     * @private
     */
    this.stageLeft_ = 0;

    /**
     * The index of the currently focused item. (-1 based)
     * @type {number}
     * @private
     */
    this.focusIndex_ = 0;

    /**
     * The time (in ms) for transitions.
     * @type {number}
     * @private
     */
    this.transitionTime_ = 600;

    /**
     * Whether or not the stage currently has CSS3 transitions set.
     * @type {boolean}
     * @private
     */
    this.hasCssTransitions_ = false;

    /**
     * The slider to be used when touch events are supported.
     * @type {?goog.fx.Dragger}
     * @private
     */
    this.slider_ = null;

    /**
     * An array of carousel items FastButtons.
     * @type {Array.<pricecheck.ui.FastButton>}
     * @private
     */
    this.itemButtons_ = [];

};
goog.inherits(pricecheck.ui.carousel.Carousel, goog.ui.Component);


/**
 * @inheritDoc
 */
pricecheck.ui.carousel.Carousel.prototype.createDom = function() {
    pricecheck.ui.carousel.Carousel.superClass_.createDom.call(this);
    this.setElementInternal(goog.soy.renderAsElement(
        pricecheck.ui.carousel.template.Carousel, {
            items: this.items_,
            moreLink: this.moreLink_}));
};


/**
 * @inheritDoc
 */
pricecheck.ui.carousel.Carousel.prototype.enterDocument = function() {
    pricecheck.ui.carousel.Carousel.superClass_.enterDocument.call(this);

    this.linkEl_ = goog.dom.getElementByClass(
        goog.getCssName('featured-description-link'), this.element_);
    this.linkButton_ = new pricecheck.ui.FastButton(this.linkEl_);

    this.nameEl_ = goog.dom.getElementByClass(
        goog.getCssName('featured-description-name'), this.element_);
    this.priceEl_ = goog.dom.getElementByClass(
        goog.getCssName('featured-description-value'), this.element_);

    this.stageEl_ = goog.dom.getElementByClass(
        goog.getCssName('featured-stage'), this.element_);

    this.itemEls_ = goog.dom.getElementsByClass(
        goog.getCssName('featured-item'), this.stageEl_);

    this.calculateSizes_();

    // If the browser supports touch events, load and listen to the slider
    if (pricecheck.device.getSupport().TOUCH_EVENTS) {
        this.slider_ = new goog.fx.Dragger(this.stageEl_, null,
            new goog.math.Rect(null, 0, null, 0));
        this.getHandler().listen(this.slider_, goog.fx.Dragger.EventType.END,
            this.onDragEnd_);
    }

    // Create an array of carousel item FastButtons
    goog.structs.forEach(this.itemEls_, function(item) {
        this.itemButtons_.push(new pricecheck.ui.FastButton(item, false));
    }, this);

    // Create "Show all" FastButton
    this.showAllButton_ = new pricecheck.ui.FastButton(
        goog.dom.getElementByClass(goog.getCssName('featured-item-more'),
            this.stageEl_));

    // Handle item click events
    goog.structs.forEach(this.itemButtons_, function(button, index) {
        this.getHandler().listen(button, 'click', function(e) {
            if (goog.dom.classes.has(button.getElement(),
                goog.getCssName('featured-item-focus'))) {
                // If focused, go to link
                window.location = '#product/' +
                    goog.dom.dataset.get(button.getElement(), 'id');
            } else {
                // Otherwise, snap to element
                e.preventDefault();
                this.snapTo(index - 1);
            }
        });
    }, this);

    // Handle "Show all" click event
    this.getHandler().listen(this.showAllButton_, 'click', function(e) {
        window.location = goog.dom.dataset.get(e.currentTarget.getElement(),
            'link');
    });

    // Window resize handler
    this.getHandler().listen(window, 'resize', function(e) {
        this.calculateSizes_();
        this.snapTo(this.focusIndex_);
    });

    // Snap to the initial item
    this.snapTo(-1);
};


/**
 * @inheritDoc
 */
pricecheck.ui.carousel.Carousel.prototype.disposeInternal = function() {
    if (this.isInDocument()) {
        this.linkButton_.dispose();
        this.showAllButton_.dispose();
        if (this.slider_) {
            this.slider_.dispose();
        }
        goog.structs.forEach(this.itemButtons_, function(button) {
            button.dispose();
        });
    }
    pricecheck.ui.carousel.Carousel.superClass_.disposeInternal.call(this);
};


/**
 * Snap the stage to an item in the carousel.
 * @param {number} index The index of the item to snap to (-1 based).
 */
pricecheck.ui.carousel.Carousel.prototype.snapTo = function(index) {
    // Ensure index within bounds
    if (index < -1) {
        index = -1;
    }
    // Index is -1 based, so length - 2 is the actual position
    if (index > this.itemEls_.length - 2) {
        index = this.itemEls_.length - 2;
    }

    if (this.focusIndex_ != index) {
        // Update the focused element, apply focus class and create FastButton
        goog.dom.classes.remove(this.getFocusedItem(),
            goog.getCssName('featured-item-focus'));
        this.focusIndex_ = index;
        goog.dom.classes.add(this.getFocusedItem(),
            goog.getCssName('featured-item-focus'));
    }

    this.stageLeft_ = -(this.focusIndex_ * this.itemWidth_);

    // If CSS transitions are supported use them, otherwise use DOM manipulation
    if (pricecheck.device.getSupport().CSS3_TRANSITIONS) {
        this.setCssTransitions_(true);
        goog.style.setPosition(this.stageEl_, this.stageLeft_);
        var delay = new goog.async.Delay(
            goog.bind(this.setCssTransitions_, this, false), 600);
        delay.start();
    } else {
        var anim = new goog.fx.dom.SlideFrom(this.stageEl_,
            [this.stageLeft_, 0], this.transitionTime_, goog.fx.easing.easeOut);
        anim.play();
    }

    // Populate price and description
    var item = this.getFocusedItem();
    goog.dom.setTextContent(this.nameEl_,
        goog.dom.dataset.get(item, 'name') || '');
    goog.dom.setTextContent(this.priceEl_,
        pricecheck.currency.format(goog.dom.dataset.get(item, 'price') || ''));
    goog.dom.setProperties(this.linkEl_, {'href': '#product/' +
        goog.dom.dataset.get(item, 'id')});

};


/**
 * Set whether to use CSS transitions or not on the stage element. We disable
 * transitions on drag end as they interfere with the dragger which updates the
 * left property in real time.
 * @param {boolean} bl Whether to enable or disable CSS transitions.
 * @private
 */
pricecheck.ui.carousel.Carousel.prototype.setCssTransitions_ = function(bl) {
    this.hasCssTransitions = bl;
    var transitionSeconds = this.transitionTime_ / 1000;
    if (bl) {
        goog.style.transition.set(this.stageEl_,
            'left ' + transitionSeconds + 's ease-out 0s');
    } else {
        goog.style.transition.removeAll(this.stageEl_);
    }
};


/**
 * Return the currently focused item.
 * @return {Element} The focused item element.
 */
pricecheck.ui.carousel.Carousel.prototype.getFocusedItem = function() {
    return this.itemEls_[this.focusIndex_ + 1];
};


/**
 * The callback for drag end events.
 * @param {goog.events.Event} e The event.
 * @private
 */
pricecheck.ui.carousel.Carousel.prototype.onDragEnd_ = function(e) {
    e.stopPropagation();
    // Whether the user swiped to the left or the right
    var gesture = (e.left <= this.stageLeft_) ? 'right' : 'left';

    // Distance dragged
    var distance = Math.abs(this.diff_(e.left, this.stageLeft_));

    // If we haven't dragged the stage atleast 10% of the item width,
    // revert the move and do nothing.
    var relativeMove = distance / this.itemWidth_;
    if (relativeMove <= 0.1) {
        this.snapTo(this.focusIndex_);
        return;
    } else {
        // TODO if dropped within 10% of an item snap to it.
        var remainder = relativeMove - Math.floor(relativeMove);
        var itemsMoved = Math.ceil(relativeMove);
        itemsMoved = (gesture == 'left') ? -itemsMoved : itemsMoved;
        this.snapTo(this.focusIndex_ + itemsMoved);
    }
};


/**
 * Calculate the item and stage widths.
 * @private
 */
pricecheck.ui.carousel.Carousel.prototype.calculateSizes_ = function() {
    this.itemWidth_ = this.itemEls_[0].clientWidth;
};


/**
 * Calcualte the difference between two integers.
 * @param {number} a The first number to compare.
 * @param {number} b The second number to compare.
 * @return {number} The difference bewteen a and b.
 * @private
 */
pricecheck.ui.carousel.Carousel.prototype.diff_ = function(a, b) {
    var absA = Math.abs(a);
    var absB = Math.abs(b);
    return (absA >= absB) ? a - b : b - a;
};
