goog.provide('pricecheck.ui.FastButton');

goog.require('goog.events.EventTarget');

goog.require('pricecheck.clickbuster');



/**
 * A button that responds to touch start and touch end events in order to reduce
 * the latency of tap events.
 * https://developers.google.com/mobile/articles/fast_buttons
 * @param {Element} element The button element.
 * @param {boolean=} opt_stopPropagation Whether we stop propagation on
 *     touchStart or not - should always stop to prevent multiple dispatches.
 * @constructor
 * @extends {goog.events.EventTarget}
 */
pricecheck.ui.FastButton = function(element, opt_stopPropagation) {
    goog.events.EventTarget.call(this);

    /**
     * The button element.
     * @type {Element}
     * @private
     */
    this.element_ = element;

    /**
     * The event handler.
     * @type {goog.events.EventHandler}
     * @private
     */
    this.handler_ = new goog.events.EventHandler(this);

    /**
     * Whether we the stop propagation on touchStart or not.
     * @type {boolean}
     * @private
     */
    this.stopPropagation_ = goog.isBoolean(opt_stopPropagation) ?
        opt_stopPropagation : true;

    // Start listening on touchstart (touch enabled devices) and click (WP 7.5,
    // Blackberry's with cursors, etc.)
    this.handler_
        .listen(this.element_, 'touchstart',
            goog.bind(this.onTouchStart_, this), false)
        .listen(this.element_, 'click', goog.bind(this.onClick_, this), false);
};
goog.inherits(pricecheck.ui.FastButton, goog.events.EventTarget);


/**
 * @inheritDoc
 */
pricecheck.ui.FastButton.prototype.disposeInternal = function() {
    pricecheck.ui.FastButton.superClass_.disposeInternal.call(this);

    this.handler_.dispose();
};


/**
 * Return the decorated link/button element.
 * @return {Element} The decorated element.
 */
pricecheck.ui.FastButton.prototype.getElement = function() {
    return this.element_;
};


/**
 * Touch start event handler.
 * @param {goog.events.BrowserEvent} e The event.
 * @private
 */
pricecheck.ui.FastButton.prototype.onTouchStart_ = function(e) {
    if (this.stopPropagation_) {
        e.stopPropagation();
    }

    this.handler_
        .listen(this.element_, 'touchend', this.onClick_, false, this)
        .listen(document.body, 'touchmove', this.onTouchMove_, false, this);

    this.startX = e.getBrowserEvent().touches[0].clientX;
    this.startY = e.getBrowserEvent().touches[0].clientY;
};


/**
 * Touch move event handler.
 * @param {goog.events.BrowserEvent} e The event.
 * @private
 */
pricecheck.ui.FastButton.prototype.onTouchMove_ = function(e) {
    if (Math.abs(e.getBrowserEvent().touches[0].clientX - this.startX) > 10 ||
        Math.abs(e.getBrowserEvent().touches[0].clientY - this.startY) > 10) {
        this.reset_();
    }
};


/**
 * The simulated click event handler (handles click and touch-end events).
 *
 * We can't reliably use window.open so we opt to use target="_blank" on links
 * to force a new window/tab.
 *
 * This means that we need to let the browser native click handling take place,
 * so we still incur the 300ms touch latency penalty.
 *
 * Blackberry has some edge cases however. In native apps, target="_blank"
 * doesn't open the browser but instead  "canibilizes" the webview. So as a
 * workaround we invoke native code to force the browser to open.
 *
 * In the browser on Blackberry 6 devices, target="_blank" has no effect and no
 * link will open. So for these devices we have to set window.location manually.
 *
 * @param {goog.events.BrowserEvent} e The event.
 * @private
 */
pricecheck.ui.FastButton.prototype.onClick_ = function(e) {
	
    // Handle target blank links - for iPhone, Android and WP7 allow the normal
    // browser behaviour as we can't use window.open etc.
    // For Blackberry, we can force a new browser using a proprietry call.
    // TODO Blackberry 7 needs to return also
    if((this.getElement().target == '_system') || (this.getElement().target == '_blank')) {
		
		if(this.getElement().target == '_blank'){

		    e.preventDefault();
		    e.stopPropagation();
			window.open(this.getElement().href, '_system');
		}
		
    	return;
    }

    e.preventDefault();
    e.stopPropagation();

    this.reset_();

    if (e.type == 'touchend') {
        pricecheck.clickbuster.preventGhostClick(this.startX, this.startY);
    }

    if (goog.isDef(this.getElement().href) && this.getElement().href) {
        this.openLink_(
            this.getElement().href,
            (this.getElement().target == '_blank'));
    } else {
        this.dispatchEvent('click');
    }
};


/**
 * Navigate to a link.
 * @param {string} link The link to navigate to.
 * @param {boolean} blank Whether to open the link in a new tab/window or not.
 * @private
 */
pricecheck.ui.FastButton.prototype.openLink_ = function(link, blank) {
    // For blackberry devices, invoke the native browser.
    if (blank) {
    } else {
        window.location.href = link;
    }
};


/**
 * @private
 */
pricecheck.ui.FastButton.prototype.reset_ = function() {
    this.handler_.unlisten(this.element_, 'touchend', this.onClick_);
    this.handler_.unlisten(document.body, 'touchmove', this.onTouchMove_);
};
