/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.ui.list.List');

goog.require('goog.Uri.QueryData');
goog.require('goog.array');
goog.require('goog.dom.classes');
goog.require('goog.string');
goog.require('goog.ui.Component');

goog.require('pricecheck.net.Xhr');
goog.require('pricecheck.ui.list.template');



/**
 * A simple list component.
 *
 * @param {goog.dom.DomHelper=} opt_domHelper Optional DOM helper.
 * @extends {goog.ui.Component}
 * @constructor
 */
pricecheck.ui.list.List = function(opt_domHelper) {
    goog.ui.Component.call(this, opt_domHelper);

    /**
     * The current page number.
     * @type {number}
     * @private
     */
    this.page_ = 1;

    /**
     * The number of items to show per page.
     * @type {number}
     * @private
     */
    this.itemsPerPage_ = 10;

    /**
     * Whether the list is a dynamic or not. Dynamic lists use XHR to fetch
     * their data and populate themselves.
     * @type {boolean}
     * @private
     */
    this.dynamic_ = false;

    /**
     * The URL to use if the list is dynamic.
     * @type {?string}
     * @private
     */
    this.url_ = null;

    /**
     * An object containing key value paris of GET params to be added to the
     * URL when making an XHR request.
     * @type {Object}
     * @private
     */
    this.params_ = {};

    /**
     * The renderer to use when creating items.
     * @type {?function(new:goog.ui.Component)}
     * @private
     */
    this.renderer_ = null;

    /**
     * XHR fetching state.
     * @type {boolean}
     * @private
     */
    this.fetching_ = false;

    /**
     * Loading element for dynamic lists.
     * @type {?Element}
     * @private
     */
    this.loadingEl_ = null;

};
goog.inherits(pricecheck.ui.list.List, goog.ui.Component);


/**
 * @inheritDoc
 */
pricecheck.ui.list.List.prototype.createDom = function() {
    pricecheck.ui.list.List.superClass_.createDom.call(this);

    goog.dom.classes.add(this.element_, goog.getCssName('list'));

    this.loadingEl_ = soy.renderAsElement(
        pricecheck.ui.list.template.Loading, {});

    if (this.isDynamic()) {
        goog.dom.appendChild(this.element_, this.loadingEl_);
        this.fetch();
    }
};


/**
 * @inheritDoc
 */
pricecheck.ui.list.List.prototype.enterDocument = function() {
    pricecheck.ui.list.List.superClass_.enterDocument.call(this);

};


/**
 * Add items to the list.
 * @param {Array.<Object>} items An array of objects to pass to the renderer
 *     and append to the list.
 */
pricecheck.ui.list.List.prototype.appendItems = function(items) {
    if (!this.renderer_) {
        return;
    }
    goog.array.forEach(items, function(item) {
        var itemComponent = new this.renderer_(item);
        this.addChild(itemComponent, true);
    }, this);
};


/**
 * Return the numebr of items being displayed on each page.
 * @return {number} The number of items per page.
 */
pricecheck.ui.list.List.prototype.getItemsPerPage = function() {
    return this.itemsPerPage_;
};


/**
 * Set the number of items being displayed on each page.
 * @param {number} itemsPerPage The number of items per page.
 */
pricecheck.ui.list.List.prototype.setItemPerPage = function(itemsPerPage) {
    this.itemsPerPage_ = itemsPerPage;
};


/**
 * Return the base URL used if this list is dynamic.
 * @return {?string} The URL.
 */
pricecheck.ui.list.List.prototype.getUrl = function() {
    return this.url_;
};


/**
 * Set the URL used if this list is dynamic.
 * @param {string} url The url to be used by the XHR.
 */
pricecheck.ui.list.List.prototype.setUrl = function(url) {
    this.url_ = url;
};


/**
 * Return the current page number.
 * @return {number} The page number.
 */
pricecheck.ui.list.List.prototype.getPage = function() {
    return this.page_;
};


/**
 * Set the current page number.
 * @param {number} page The page number.
 */
pricecheck.ui.list.List.prototype.setPage = function(page) {
    this.page_ = page;
};


/**
 * Return the GET params to be added to the URL for XHR requests.
 * @return {Object} Object of key value parameters.
 */
pricecheck.ui.list.List.prototype.getParams = function() {
    return this.params_;
};


/**
 * Set the GET params to be added to the URL for XHR requests.
 * @param {Object} params Object of key value parameters.
 */
pricecheck.ui.list.List.prototype.setParams = function(params) {
    this.params_ = params;
};


/**
 * Return whether the list is dynamic or not.
 * @return {boolean} Whether the list is dynamic or not.
 */
pricecheck.ui.list.List.prototype.isDynamic = function() {
    return this.dynamic_;
};


/**
 * Set whether the list is dynamic or not.
 * @param {boolean} dynamic Whether the list is dynamic or not.
 */
pricecheck.ui.list.List.prototype.setDynamic = function(dynamic) {
    this.dynamic_ = dynamic;
};


/**
 * Return the render currently in use by the list.
 * @return {?function(new:goog.ui.Component)} The render being used.
 */
pricecheck.ui.list.List.prototype.getRenderer = function() {
    return this.renderer_;
};


/**
 * Set the renderer to be used when rendering items.
 * @param {function (new:goog.ui.Component)} renderer The renderer to use.
 */
pricecheck.ui.list.List.prototype.setRenderer = function(renderer) {
    this.renderer_ = renderer;
};


/**
 * If the list is dynamic, invoke an XHR to fetch more data and append them to
 * the list.
 * @protected
 */
pricecheck.ui.list.List.prototype.fetch = function() {
    if (!this.dynamic_ || !this.url_ || this.fetching_) {
        return;
    }

    this.fetching_ = true;

    var url = this.buildXhrUrl_();
    pricecheck.net.Xhr.send(
        pricecheck.HOST + '/proxy.php?url=' + goog.string.urlEncode(url),
        null,
        goog.bind(this.onFetchSuccess, this),
        goog.bind(this.onFetchFail, this),
        'GET',
        false);
};


/**
 * Handle the XHR response. You need to override this sage the response into an
 * Array format suitable for your renderer before passing it to appendItems.
 * @param {Object|Array} response The response from the XHR request.
 * @protected
 */
pricecheck.ui.list.List.prototype.onFetchSuccess = function(response) {
    this.fetching_ = false;
    this.page_ += 1;
    goog.dom.removeNode(this.loadingEl_);
};


/**
 * Handle a failed XHR response.
 * @param {Object|Array} response The response from the XHR request.
 * @protected
 */
pricecheck.ui.list.List.prototype.onFetchFail = function(response) {
    this.fetching_ = false;
    goog.dom.removeNode(this.loadingEl_);
};


/**
 * Build the URL used for XHR requests, appending the GET and limit
 * parameters.
 * @private
 * @return {string} The URL for the XHR request.
 */
pricecheck.ui.list.List.prototype.buildXhrUrl_ = function() {
    var queryData = new goog.Uri.QueryData();
    // Add GET params
    for (var param in this.params_) {
        queryData.set(param, this.params_[param]);
    }
    // Add limit params
    queryData.set('start_num', (this.page_ - 1) * this.itemsPerPage_);
    queryData.set('count', this.itemsPerPage_);
    return this.url_ + '?' + queryData.toString();
};
