/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 * TODO Class needs refactoring - too tightly coupled to category/product
 * structure.
 */
goog.provide('pricecheck.ui.livetiles.Container');

goog.require('goog.Timer');
goog.require('goog.dom');
goog.require('goog.structs');
goog.require('goog.structs');
goog.require('goog.ui.Component');

goog.require('pricecheck.net.Xhr');
goog.require('pricecheck.ui.livetiles.tiles.Books');
goog.require('pricecheck.ui.livetiles.tiles.Catalogue');
goog.require('pricecheck.ui.livetiles.tiles.Cellphones');
goog.require('pricecheck.ui.livetiles.tiles.Computers');
goog.require('pricecheck.ui.livetiles.tiles.Insurance');
goog.require('pricecheck.ui.livetiles.tiles.Electronics');
goog.require('pricecheck.ui.livetiles.tiles.Flights');
goog.require('pricecheck.ui.livetiles.tiles.Mtn');
goog.require('pricecheck.ui.livetiles.tiles.Gaming');
goog.require('pricecheck.ui.livetiles.tiles.Tablets');
// goog.require('pricecheck.ui.livetiles.tiles.Dvd');
// goog.require('pricecheck.ui.livetiles.tiles.Music');



/**
 * @param {goog.dom.DomHelper=} opt_domHelper The DOM helper object for the
 *     document we want to render in.
 * @constructor
 * @extends {goog.ui.Component}
 */
pricecheck.ui.livetiles.Container = function(opt_domHelper) {
    goog.ui.Component.call(this, opt_domHelper);

    /**
     * The timer for flipping tiles.
     * @type {goog.Timer}
     * @private
     */
    this.timer_ = new goog.Timer();

    /**
     * The timer for returning tiles to their non-flipped state
     * @type {goog.Timer}
     * @private
     */
    this.flippedTimer_ = new goog.Timer(3000);

    /**
     * The currently flipped over tile
     * @type {?pricecheck.ui.livetiles.tiles.Abstract}
     * @private
     */
    this.tileFlipped_ = null;

};
goog.inherits(pricecheck.ui.livetiles.Container, goog.ui.Component);


/**
 * @inheritDoc
 */
pricecheck.ui.livetiles.Container.prototype.createDom = function() {
    pricecheck.ui.livetiles.Container.superClass_.createDom.call(this);

    goog.dom.classes.add(this.element_, goog.getCssName('tiles'));
};


/**
 * @inheritDoc
 */
pricecheck.ui.livetiles.Container.prototype.enterDocument = function() {
    pricecheck.ui.livetiles.Container.superClass_.enterDocument.call(this);

    this.timer_.setInterval(this.getInterval_());
    this.timer_.start();

    this.getHandler()
        .listen(this.timer_, goog.Timer.TICK, this.flip_)
        .listen(this.flippedTimer_, goog.Timer.TICK, this.flipBack_)
        .listen(document, 'pause', function(e) {
            this.timer_.stop();
            this.flippedTimer_.stop();
        })
        .listen(document, 'resume', function(e) {
            this.timer_.start();
            this.flippedTimer_.start();
        });
};


/**
 * @inheritDoc
 */
pricecheck.ui.livetiles.Container.prototype.disposeInternal = function() {
    pricecheck.ui.livetiles.Container.superClass_.disposeInternal.call(this);

    this.timer_.dispose();
    this.flippedTimer_.dispose();
};


/**
 * Add tiles to the container.
 * @param {Array.<pricecheck.ui.livetiles.tiles.Abstract>} tiles A collection of
 *     to add.
 */
pricecheck.ui.livetiles.Container.prototype.addTiles = function(tiles) {
    goog.structs.forEach(tiles, function(tile) {
        this.addChild(tile, true);
    }, this);

    // Fetch popular products to display on the reverse side of each tile if
    // we're online.
    if (pricecheck.isOnLine()) {
        var url = pricecheck.API_HOST +
                '/crud/entity/product/get_popular_front_page?format=json&appversion=za-mtn-android-1.6.2';
        goog.structs.forEach(this.getChildIds(), function(id) {
            if (id) {
                url += '&category_ids[]=' + id;
            }
        });
        pricecheck.net.Xhr.send(
            pricecheck.HOST + '/proxy.php?url=' + goog.string.urlEncode(url),
            null,
            goog.bind(this.onFetchSuccess_, this),
            null,
            'GET',
            false);
    }
};


/**
 * Flip a randomly selected tile over and ensure only one tile is flipped at any
 * one time.
 * @private
 */
pricecheck.ui.livetiles.Container.prototype.flip_ = function() {
    this.timer_.stop();
    this.flippedTimer_.start();

    // Pick a random child a flip it
    var child = this.getChildAt(Math.floor(Math.random() *
        this.getChildCount()));
    child.flip();
    this.tileFlipped_ =
        /** @type {pricecheck.ui.livetiles.tiles.Abstract} */ (child);
};


/**
 * Return the flipped tile to it's non-flipped state.
 * @private
 */
pricecheck.ui.livetiles.Container.prototype.flipBack_ = function() {

    this.flippedTimer_.stop();
    this.timer_.start();

    // Make sure we have a flipped tile
    if (!this.tileFlipped_) {
        return;
    }

    this.tileFlipped_.flip();
    this.tileFlipped_ = null;
};


/**
 * The interval between showing the reverse of a tile.
 * @return {number} The interval.
 * @private
 */
pricecheck.ui.livetiles.Container.prototype.getInterval_ = function() {
    return 4000 /* 4 seconds */;
};


/**
 * The category popular products XHR callback.
 * @param {Object} response The XHR response data.
 * @private
 */
pricecheck.ui.livetiles.Container.prototype.onFetchSuccess_ =
    function(response) {

    goog.structs.forEach(response['results'], function(result) {
        var child = this.getChild(result['category']['id']);
        if (child) {
            child.setReverseContent(result['products'][0]);
        }
    }, this);
};


/**
 * Static facory for returning the default list of tiles.
 * @return {Array.<pricecheck.ui.livetiles.tiles.Abstract>} An array of tiles.
 */
pricecheck.ui.livetiles.Container.defaultTiles = function() {
    return [
        new pricecheck.ui.livetiles.tiles.Cellphones(),
        new pricecheck.ui.livetiles.tiles.Tablets(),
        new pricecheck.ui.livetiles.tiles.Computers(),
        new pricecheck.ui.livetiles.tiles.Electronics(),
        new pricecheck.ui.livetiles.tiles.Mtn(),
        new pricecheck.ui.livetiles.tiles.Catalogue(),
        new pricecheck.ui.livetiles.tiles.Gaming(),
        new pricecheck.ui.livetiles.tiles.Insurance(),
        new pricecheck.ui.livetiles.tiles.Flights()
    ];
};
