/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.ui.livetiles.tiles.Abstract');

goog.require('goog.dom');
goog.require('goog.dom.classes');
goog.require('goog.fx.css3.Transition');
goog.require('goog.style');
goog.require('goog.ui.Component');

goog.require('pricecheck.ui.FastButton');
goog.require('pricecheck.ui.livetiles.tiles.template');



/**
 * @param {goog.dom.DomHelper=} opt_domHelper The DOM helper object for the
 *     document we want to render in.
 * @constructor
 * @extends {goog.ui.Component}
 */
pricecheck.ui.livetiles.tiles.Abstract = function(opt_domHelper) {
    goog.ui.Component.call(this, opt_domHelper);

    /**
     * The class name for the icon to display.
     * @type {?string}
     * @protected
     */
    this.imageClass = null;

    /**
     * The link the tile points to.
     * @type {?string}
     * @protected
     */
    this.link = null;

    /**
     * Whether this link points to in external resource and must open in a new
     * tab.
     * @type {boolean}
     * @protected
     */
    this.isExternal = false;

    /**
     * The text content of the tile.
     * @type {?string}
     * @protected
     */
    this.text = null;

    /**
     * Whether or not the tile has reverse content or not.
     * @type {boolean}
     * @private
     */
    this.hasReverseContent_ = false;

    /**
     * The element to move upwards to "flip" the tile.
     * @type {?Element}
     * @private
     */
    this.tileChildrenEl_ = null;

    /**
     * Whether the tile is currently flipped or not.
     * @type {boolean}
     * @private
     */
    this.flipped_ = false;

    /**
     * The reverse tile product description.
     * @type {?Element}
     * @private
     */
    this.productNameEl_ = null;

    /**
     * The reverse tile product price.
     * @type {?Element}
     * @private
     */
    this.productPriceEl_ = null;

    /**
     * The FastButton of the tile.
     * @type {?pricecheck.ui.FastButton}
     * @private
     */
    this.button_ = null;

};
goog.inherits(pricecheck.ui.livetiles.tiles.Abstract, goog.ui.Component);


/**
 * @inheritDoc
 */
pricecheck.ui.livetiles.tiles.Abstract.prototype.createDom = function() {
    pricecheck.ui.livetiles.tiles.Abstract.superClass_.createDom.call(this);

    goog.dom.classes.add(this.element_, goog.getCssName('tile-container'));

    goog.dom.appendChild(this.element_, soy.renderAsElement(
        pricecheck.ui.livetiles.tiles.template.Tile, {
            imageClass: this.imageClass,
            link: this.link,
            external: this.isExternal,
            text: this.text}));

    this.tileChildrenEl_ = goog.dom.getElementByClass(
        goog.getCssName('tile-children-container'), this.element_);

    this.productNameEl_ = goog.dom.getElementByClass(
        goog.getCssName('tile-product-name'), this.tileChildrenEl_);

    this.productPriceEl_ = goog.dom.getElementByClass(
        goog.getCssName('tile-product-price'), this.tileChildrenEl_);
};


/**
 * @inheritDoc
 */
pricecheck.ui.livetiles.tiles.Abstract.prototype.enterDocument = function() {
    pricecheck.ui.livetiles.tiles.Abstract.superClass_.enterDocument.call(this);

    this.button_ = new pricecheck.ui.FastButton(
        goog.dom.getElementsByTagNameAndClass('a', null, this.element_)[0]);
};


/**
 * @inheritDoc
 */
pricecheck.ui.livetiles.tiles.Abstract.prototype.disposeInternal = function() {
    if (this.isInDocument()) {
        this.button_.dispose();
    }
    pricecheck.ui.livetiles.tiles.Abstract.superClass_.disposeInternal.call(
        this);
};


/**
 * Toggle the flipped state of the tile.
 */
pricecheck.ui.livetiles.tiles.Abstract.prototype.flip = function() {
    // Only flip tiles that have reverse content
    if (!this.hasReverseContent_) {
        return;
    }
    if (this.flipped_) {
        var transition = new goog.fx.css3.Transition(this.tileChildrenEl_, 0.5,
            null, {'margin-top': '0'}, {
                property: 'all',
                duration: 0.5,
                timing: 'linear',
                delay: 0
        });
        transition.play();
    } else {
        var transition = new goog.fx.css3.Transition(this.tileChildrenEl_, 0.5,
            {'margin-top': '0'}, {'margin-top': '-100%'}, {
                property: 'all',
                duration: 0.5,
                timing: 'linear',
                delay: 0
            });
        transition.play();
    }
    this.flipped_ = !this.flipped_;
};


/**
 * Set the content on the reverse side of the the tile.
 * @param {Object} content A content object map to populate the tile with.
 */
pricecheck.ui.livetiles.tiles.Abstract.prototype.setReverseContent =
    function(content) {

    if (typeof(content) != 'undefined' && 'price' in content &&
        'name' in content) {
        this.hasReverseContent_ = true;
        goog.dom.setTextContent(this.productNameEl_, content['name']);
        goog.dom.setTextContent(this.productPriceEl_, 'R' + content['price']);
    }
};
