/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.ui.page.header.Header');

goog.require('goog.dom');
goog.require('goog.dom.dataset');
goog.require('goog.dom.forms');
goog.require('goog.string');
goog.require('goog.structs');
goog.require('goog.style');
goog.require('goog.ui.Component');
goog.require('goog.ui.LabelInput');

goog.require('pricecheck.ui.FastButton');
goog.require('pricecheck.ui.page.header.template');



/**
 * @param {goog.dom.DomHelper=} opt_domHelper The DOM helper object for the
 *     document we want to render in.
 * @constructor
 * @extends {goog.ui.Component}
 */
pricecheck.ui.page.header.Header = function(opt_domHelper) {
    goog.ui.Component.call(this, opt_domHelper);

    /**
     * Whether the search bar is display or not.
     * @type {boolean}
     * @private
     */
    this.showSearch_ = false;

    /**
     * An array of FastButtons decorated on this page so we can dispose them
     * cleanly.
     * @type Array.<pricecheck.ui.FastButton>
     * @private
     */
    this.buttons_ = [];
};
goog.inherits(pricecheck.ui.page.header.Header, goog.ui.Component);


/**
 * @inheritDoc
 */
pricecheck.ui.page.header.Header.prototype.createDom = function() {
    pricecheck.ui.page.header.Header.superClass_.createDom.call(this);

    this.setElementInternal(
        soy.renderAsElement(pricecheck.ui.page.header.template.Header, {}));

};


/**
 * @inheritDoc
 */
pricecheck.ui.page.header.Header.prototype.enterDocument = function() {
    pricecheck.ui.page.header.Header.superClass_.enterDocument.call(this);

    this.backButtonEl_ = goog.dom.getElementByClass(
        goog.getCssName('header-back-button'), this.element_);
    var backButton = new pricecheck.ui.FastButton(this.backButtonEl_);
    this.buttons_.push(backButton);

    this.searchButtonEl_ = goog.dom.getElement('search');
    var searchButton = new pricecheck.ui.FastButton(this.searchButtonEl_);
    this.buttons_.push(searchButton);

    this.menuButtonEl_ = goog.dom.getElement('menu');
    var menuButton = new pricecheck.ui.FastButton(this.menuButtonEl_);
    this.buttons_.push(menuButton);

    var logoButton = new pricecheck.ui.FastButton(
        goog.dom.getElementByClass(goog.getCssName('header-logo-container'),
            this.element_));
    this.buttons_.push(logoButton);

    this.searchBarEl_ = goog.dom.getElementByClass(
        goog.getCssName('search-bar'), this.element_);

    var searchFormEl = goog.dom.getElementsByTagNameAndClass('form', null,
        this.searchBarEl_)[0];

    this.searchInputEl_ = goog.dom.getElementByClass(
        goog.getCssName('search-bar-input'), searchFormEl);
    var searchLabel = new goog.ui.LabelInput('type product name here');
    searchLabel.decorate(this.searchInputEl_);

    this.getHandler()
        .listen(searchButton, 'click', function(e) {
            e.preventDefault();
            this.showSearch(!this.showSearch_);
        })
        .listen(searchFormEl, 'submit', function(e) {
            e.preventDefault();
            var term = goog.dom.forms.getValue(this.searchInputEl_).trim();
            if (term != '') {
                window.location.href = '#search/' + term;
            }
        });
};


/**
 * @inheritDoc
 */
pricecheck.ui.page.header.Header.prototype.disposeInternal = function() {
    if (this.isInDocument()) {
        goog.structs.forEach(this.buttons_, function(button) {
            button.dispose();
        });
    }
    pricecheck.ui.page.header.Header.superClass_.disposeInternal.call(this);
};


/**
 * Reset the search bar. Show all buttons and hide the search bar.
 */
pricecheck.ui.page.header.Header.prototype.reset = function() {
    this.setSearchText('');
    this.showSearch(false);
    this.showBackButton(true);
    this.showSearchButton(true);
};


/**
 * Show or hide the search bar.
 * @param {boolean} bl Whether to hide or display the search bar.
 */
pricecheck.ui.page.header.Header.prototype.showSearch = function(bl) {
    this.showSearch_ = bl;
    goog.style.showElement(this.searchBarEl_, bl);
};


/**
 * Show or hide the search button.
 * @param {boolean} bl Whether to hide or display the search button.
 */
pricecheck.ui.page.header.Header.prototype.showSearchButton = function(bl) {
    this.setVisible(this.searchButtonEl_, bl);
};


/**
 * Show or hide the back button.
 * @param {boolean} bl Whether to hide or display the back button.
 */
pricecheck.ui.page.header.Header.prototype.showBackButton = function(bl) {
    this.setVisible(this.backButtonEl_, bl);
};


/**
 * Set the search field text.
 * @param {string} search The text to display in the search field.
 */
pricecheck.ui.page.header.Header.prototype.setSearchText = function(search) {
    goog.dom.forms.setValue(this.searchInputEl_, search);
};


/**
 * Hide an element using the visibility property rather the the display
 * property. Maintains it's DOM structure and layout.
 * @param {Element} el The element to hide or show.
 * @param {boolean} bl Whether to hide or show the element.
 */
pricecheck.ui.page.header.Header.prototype.setVisible = function(el, bl) {
    if (bl) {
        goog.style.setStyle(el, 'visibility', 'visible');
    } else {
        goog.style.setStyle(el, 'visibility', 'hidden');
    }
};
