/**
 * Copyright 2012 PriceCheck
 * @author Rob Hofmeyr
 */
goog.provide('pricecheck.ui.page.product.Product');

goog.require('goog.Timer');
goog.require('goog.array');
goog.require('goog.dom');
goog.require('goog.dom.classes');
goog.require('goog.i18n.NumberFormat');
goog.require('goog.string');
goog.require('goog.structs');
goog.require('goog.style');
goog.require('goog.ui.Component');
goog.require('goog.ui.Dialog');
goog.require('pricecheck.db.Favorites');
goog.require('pricecheck.ui.FastButton');
goog.require('pricecheck.ui.dialog.share.Share');
goog.require('pricecheck.ui.dialog.sort.Event');
goog.require('pricecheck.ui.dialog.sort.Sort');
goog.require('pricecheck.ui.page.product.tabs.Details');
goog.require('pricecheck.ui.page.product.tabs.Offers');
goog.require('pricecheck.ui.page.product.tabs.Reviews');
goog.require('pricecheck.ui.page.product.template');
goog.require('pricecheck.ui.tabs.Event');
goog.require('pricecheck.ui.tabs.Tabs');



/**
 * @param {number} productId The product ID of the product.
 * @param {goog.dom.DomHelper=} opt_domHelper The DOM helper object for the
 *     document we want to render in.
 * @constructor
 * @extends {goog.ui.Component}
 */
pricecheck.ui.page.product.Product = function(productId, opt_domHelper) {
    goog.ui.Component.call(this, opt_domHelper);

    /**
     * The product ID of the product.
     * @type {number}
     * @private
     */
    this.productId_ = parseInt(productId, 10);
    if (!goog.isNumber(this.productId_)) {
        throw 'Invalid Product Id';
    }

    /**
     * The XHR response object.
     * @type {Object?}
     * @private
     */
    this.product_ = null;

    /**
     * The currently displayed tab component.
     * @type {goog.ui.Component}
     * @private
     */
    this.currentContentComponent_ = null;

    /**
     * The favorites localStorage db abstraction
     * @type {pricecheck.db.Favorites}
     * @private
     */
    this.favoriteDb_ = pricecheck.db.Favorites.getInstance();

    /**
     * Whether the current product is favorites or not.
     * @type {boolean}
     * @private
     */
    this.isFavorite_ = this.favoriteDb_.exists(this.productId_);

    /**
     * An array of FastButtons decorated on this page so we can dispose them
     * cleanly.
     * @type Array.<pricecheck.ui.FastButton>
     * @private
     */
    this.buttons_ = [];

    /**
     * An array of expert reviews.
     * @type Array
     * @private
     */
    this.expertReviews_ = null;

};
goog.inherits(pricecheck.ui.page.product.Product, goog.ui.Component);


/**
 * @inheritDoc
 */
pricecheck.ui.page.product.Product.prototype.enterDocument = function() {
    pricecheck.ui.page.product.Product.superClass_.enterDocument.call(this);

    // Try find the item in localStorage first
    if (this.isFavorite_) {
        var response = {'results': this.favoriteDb_.findById(this.productId_)};
        this.onProductSuccess(response);
    // Otherwise fetch the item via Xhr
    } else {
        var url = pricecheck.API_HOST +
                '/crud/entity/product/get_product_by_id?id=' +
                this.productId_ + '&format=json&appversion=za-mtn-android-1.6.2';

        pricecheck.net.Xhr.send(
            pricecheck.HOST + '/proxy.php?url=' + goog.string.urlEncode(url),
            null,
            goog.bind(this.onProductSuccess, this),
            null,
            'GET',
            true);
    }
};


/**
 * @inheritDoc
 */
pricecheck.ui.page.product.Product.prototype.disposeInternal = function() {
    if (this.tabs_) {
        this.tabs_.dispose();
    }
    if (this.buttons_) {
        goog.structs.forEach(this.buttons_, function(button) {
            button.dispose();
        });
    }
    this.favoriteDb_.dispose();
    pricecheck.ui.page.product.Product.superClass_.disposeInternal.call(this);
};


/**
 * The XHR success handler. Store and render the response.
 * @param {Object} response The XHR response.
 * @protected
 */
pricecheck.ui.page.product.Product.prototype.onProductSuccess =
    function(response) {

    this.product_ = response['results'];

    soy.renderElement(this.element_,
        pricecheck.ui.page.product.template.Product, {
            product: this.product_,
            priceMin: pricecheck.currency.format(this.product_['price_min']),
            priceMax: pricecheck.currency.format(this.product_['price_max']),
            isFavorite: this.isFavorite_
    });

    var ratingEl = goog.dom.getElementByClass(
        goog.getCssName('offer-detail-rating'), this.element_);
    this.ratingButton_ = new pricecheck.ui.FastButton(ratingEl);
    this.buttons_.push(this.ratingButton_);

    var favoriteEl = goog.dom.getElementByClass(
        goog.getCssName('offer-detail-favorites-button'), this.element_);
    this.favoriteButton_ = new pricecheck.ui.FastButton(favoriteEl);
    this.buttons_.push(this.favoriteButton_);

    var imgEl = goog.dom.getElementByClass(
        goog.getCssName('offer-detail-image'), this.element_);
    this.imgButton_ = new pricecheck.ui.FastButton(imgEl);
    this.buttons_.push(this.imgButton_);

    var tabsEl = goog.dom.getElementByClass(
        goog.getCssName('tab-bar'), this.element_);
    this.tabs_ = new pricecheck.ui.tabs.Tabs();
    this.tabs_.decorate(tabsEl);

    this.tabContentEl_ = goog.dom.getElementByClass(
        goog.getCssName('tab-content'), this.element_);

    this.addListeners();

    // Get expert reviews from API
    var expertUrl = pricecheck.API_HOST +
                    '/reviews/expert?format=json&product_id=' +
                    this.productId_;

    pricecheck.net.Xhr.send(
        pricecheck.HOST + '/proxy.php?url=' + goog.string.urlEncode(expertUrl),
        null,
        goog.bind(this.onExpertSuccess, this),
        null,
        'GET',
        false);
};

/**
 */
pricecheck.ui.page.product.Product.prototype.onExpertSuccess = function(response) {

    // Assign expert review results
    this.expertReviews_ = response['results'];

    if (this.expertReviews_ != null && this.expertReviews_.length > 0) {
        this.addExpertStars();
    }
}

/**
 */
pricecheck.ui.page.product.Product.prototype.addExpertStars = function() {

    // Get normal stars dom element
    var normalStarsEl = goog.dom.getElementByClass(
        goog.getCssName('offer-detail-rating'));

    // Create expert stars dom element
    var expertStarsEl = goog.dom.createDom('div',
        {'class': goog.getCssName('offer-detail-rating expert-rating')});

    var ratingOn  = goog.getCssName('rating') + ' ' + goog.getCssName('icon-18') + ' ' + goog.getCssName('icon-rating-on');
    var ratingOff = goog.getCssName('rating') + ' ' + goog.getCssName('icon-18') + ' ' + goog.getCssName('icon-rating-off');

    // Generate full stars and append to div
    for(var i=0; i<this.expertReviews_[0]['score']; i++) {
        var itemEl = goog.dom.createDom('span', 
            {'class': ratingOn});
        goog.dom.append(expertStarsEl, itemEl);
    }

    // Generate empty stars and append to div
    for(var i=0; i<(5-this.expertReviews_[0]['score']); i++) {
        var itemEl = goog.dom.createDom('span', 
            {'class': ratingOff}, expertStarsEl);
    }

    // Insert Expert Reviews text
    goog.dom.append(expertStarsEl, goog.dom.createDom('p',undefined,'Expert Reviews'));

    // Insert User Reviews text
    goog.dom.append(normalStarsEl, goog.dom.createDom('p',undefined,'User Reviews'));

    // Insert the expert stars before the normal stars
    goog.dom.insertSiblingBefore(expertStarsEl, normalStarsEl);

    // Create button
    this.expertRatingButton_ = new pricecheck.ui.FastButton(expertStarsEl);
    this.buttons_.push(this.expertRatingButton_);

    // Handle expert rating button
    this.getHandler()
        .listen(this.expertRatingButton_, 'click', function(e) {
            e.preventDefault();
            this.tabs_.setSelectedIndex(2);
        });

    // If the reviews tab is already selected, recreate the tab controller
    if (this.tabs_.getSelected() == 'reviews') {
        this.tabController(this.tabs_.getSelected());
    }
}


/**
 * Add event handlers and listeners to the generated DOM structure and init
 * the tabs.
 * @protected
 */
pricecheck.ui.page.product.Product.prototype.addListeners = function() {

    var online = pricecheck.isOnLine();

    this.getHandler()
        .listen(this.favoriteButton_, 'click', function(e) {
            e.preventDefault();
            this.favorite_();
        })
        .listen(this.tabs_, pricecheck.ui.tabs.Event.CHANGE, function(e) {
            this.tabController(this.tabs_.getSelected());
        })
        .listen(this.tabs_, pricecheck.ui.tabs.Event.ACTION, function(e) {
            this.tabAction(e.action);
        })
        .listen(this, pricecheck.ui.dialog.sort.Event.SORTED, function(e) {
            this.tabs_.setSelectedIndex(0);
        })
        .listen(this.ratingButton_, 'click', function(e) {
            this.tabs_.setSelectedIndex(2);
        })
        .listen(this.imgButton_, 'click', function(e) {
            if (online) {
                window.location = '#image/' +
                    goog.string.urlEncode(this.product_['image_big']);
            }
        });

    // Init the first tab (Offers) unless we have a Guzzle product
    if (this.product_['num_shops'] == 1 &&
            this.product_['offers']['0']['shop_id'] == '1406') {
        this.tabs_.setSelectedIndex(1);
    } else {
        this.tabs_.setSelectedIndex(0);
    }

    // Check for online/offline
    this.tabs_.setEnabled(3, online);

};


/**
 * The handler for tab change events. Disposes any current tab and renders
 * the new tab.
 * @param {string} tab The tab to change to.
 * @protected
 */
pricecheck.ui.page.product.Product.prototype.tabController = function(tab) {
    if (this.currentContentComponent_) {
        this.currentContentComponent_.dispose();
        this.currentContentComponent_ = null;
    }

    switch (tab) {
        case 'offers':
            this.currentContentComponent_ =
                new pricecheck.ui.page.product.tabs.Offers(this.product_);
            break;
        case 'details':
            this.currentContentComponent_ =
                new pricecheck.ui.page.product.tabs.Details(this.product_);
            break;
        case 'reviews':
            this.currentContentComponent_ =
                new pricecheck.ui.page.product.tabs.Reviews(this.product_, this.expertReviews_);
            break;
    }

    if (this.currentContentComponent_) {
        this.addChild(this.currentContentComponent_);
        this.currentContentComponent_.render(this.tabContentEl_);
    }
};


/**
 * The handler for tab action events. A tab action differs from a tab change in
 * that is doesn't change the tab content component.
 * @param {string} action The tab action.
 * @protected
 */
pricecheck.ui.page.product.Product.prototype.tabAction = function(action) {
    var dialog;
    switch (action) {
        case 'share':
            dialog = new pricecheck.ui.dialog.share.Share(this.product_);
            this.addChild(dialog);
            dialog.setVisible(true);
            break;
        case 'sort':
            dialog = new pricecheck.ui.dialog.sort.Sort(this.product_);
            this.addChild(dialog);
            dialog.setVisible(true);
            break;
    }
};


/**
 * Favorite click event handler. Add this item to the favorites localStorage and
 * change the favorite link text.
 * @private
 */
pricecheck.ui.page.product.Product.prototype.favorite_ = function() {

    var text;
    if (!this.isFavorite_) {
        this.favoriteDb_.set(this.productId_, this.product_);
        this.isFavorite_ = true;
        text = 'Remove from Favourites';
    } else {
        this.favoriteDb_.remove(this.productId_);
        this.isFavorite_ = false;
        text = 'Add to Favourites';
    }

    var textEl = goog.dom.getElementByClass(
            goog.getCssName('button-grey-small-text'),
                this.favoriteButton_.getElement());

    if (pricecheck.device.getOs() == pricecheck.Device.os.BLACKBERRY) {
        goog.dom.setTextContent(textEl, text);
    } else {
        goog.style.setStyle(this.favoriteButton_.getElement(), 'opacity', 0);
        var timer = new goog.Timer(500);
        this.getHandler().listen(timer, 'tick', function(e) {
            goog.dom.setTextContent(textEl, text);
            goog.style.setStyle(this.favoriteButton_.getElement(), 'opacity',
                1);
            timer.dispose();
        }, false, this);
        timer.start();
    }

};
