goog.provide('pricecheck.ui.tabs.Event');
goog.provide('pricecheck.ui.tabs.Tabs');

goog.require('goog.array');
goog.require('goog.dom');
goog.require('goog.dom.classes');
goog.require('goog.dom.dataset');
goog.require('goog.ui.Component');

goog.require('pricecheck.ui.FastButton');



/**
 * @param {goog.dom.DomHelper=} opt_domHelper The DOM helper object for the
 *     document we want to render in.
 * @constructor
 * @extends {goog.ui.Component}
 */
pricecheck.ui.tabs.Tabs = function(opt_domHelper) {
    goog.ui.Component.call(this, opt_domHelper);

    /**
     * An array of tab <li> elements.
     * @type { {length: number} }
     * @private
     */
    this.tabs_ = [];

    /**
     * The currently selected li tab.
     * @type {Element}
     * @private
     */
    this.selectedTab_ = null;

    /**
     * An array of FastButtons decorated on this page so we can dispose them
     * cleanly.
     * @type Array.<pricecheck.ui.FastButton>
     * @private
     */
    this.tabButtons_ = [];

};
goog.inherits(pricecheck.ui.tabs.Tabs, goog.ui.Component);


/**
 * @inheritDoc
 */
pricecheck.ui.tabs.Tabs.prototype.decorateInternal = function(element) {
    pricecheck.ui.tabs.Tabs.superClass_.decorateInternal.call(this, element);

    this.tabs_ = goog.dom.getElementsByTagNameAndClass('li');
    this.selectedTab_ = goog.dom.getElementByClass(goog.getCssName('on'),
        this.element_);
};


/**
 * @inheritDoc
 */
pricecheck.ui.tabs.Tabs.prototype.enterDocument = function() {
    pricecheck.ui.tabs.Tabs.superClass_.enterDocument.call(this);

    goog.array.forEach(this.tabs_, function(el) {
        this.tabButtons_.push(new pricecheck.ui.FastButton(el));
    }, this);

    goog.array.forEach(this.tabButtons_, function(button) {
        this.getHandler().listen(button, 'click', function(e) {
            this.changeTab_(e.currentTarget.getElement());
        }, false, this);
    }, this);
};


/**
 * @inheritDoc
 */
pricecheck.ui.tabs.Tabs.prototype.disposeInternal = function() {
    if (this.isInDocument()) {
        goog.array.forEach(this.tabButtons_, function(button) {
            button.dispose();
        });
    }
    pricecheck.ui.tabs.Tabs.superClass_.disposeInternal.call(this);
};


/**
 * Return the string representation of the currently selected tab. Uses the data
 * attribute "tab-".
 * @return {?string} The selected tab text representation.
 */
pricecheck.ui.tabs.Tabs.prototype.getSelected = function() {
    if (this.selectedTab_ && goog.dom.dataset.has(this.selectedTab_, 'tab')) {
        return goog.dom.dataset.get(this.selectedTab_, 'tab');
    } else {
        return null;
    }
};


/**
 * Return the currently selected tab element.
 * @return {Element} The currently selected tab element.
 */
pricecheck.ui.tabs.Tabs.prototype.getSelectedElement = function() {
    return this.selectedTab_;
};


/**
 * Change to the tab at the specified index. 0-based.
 * @param {number} index The index of the tab to change to.
 */
pricecheck.ui.tabs.Tabs.prototype.setSelectedIndex = function(index) {
    if (index < 0 || index >= this.tabs_.length) {
        throw 'Index out of bounds.';
    }
    this.changeTab_(this.tabs_[index]);
};


/**
 * Enable or disable a tab at the specified index. 0-based.
 * @param {number} index The index of the tab to enable/disable.
 * @param {boolean} bl Whether to enable or disable the tab.
 */
pricecheck.ui.tabs.Tabs.prototype.setEnabled = function(index, bl) {
    if (bl) {
         goog.dom.classes.remove(this.tabs_[index],
            goog.getCssName('disabled'));
    } else {
        goog.dom.classes.add(this.tabs_[index], goog.getCssName('disabled'));
    }
};


/**
 * Change to the specified tab. Dispatches one of the pricecheck.ui.tabs.Event
 * events.
 * @param {HTMLLIElement} el The tab to change to.
 * @private
 */
pricecheck.ui.tabs.Tabs.prototype.changeTab_ = function(el) {

    if (goog.dom.classes.has(el, goog.getCssName('disabled'))) {
        return;
    }

    // If selectable, dispatch a CHANGE event
    if (goog.dom.classes.has(el, goog.getCssName('selectable'))) {
        if (this.selectedTab_) {
            goog.dom.classes.remove(this.selectedTab_, goog.getCssName('on'));
        }
        goog.dom.classes.add(el, goog.getCssName('on'));

        this.selectedTab_ = el;

        this.dispatchEvent(pricecheck.ui.tabs.Event.CHANGE);
    // Otherwise dispatch an ACTION event
    } else {
        this.dispatchEvent({
            'type': pricecheck.ui.tabs.Event.ACTION,
            'action': goog.dom.dataset.get(el, 'tab')
        });
    }

};


/**
 * An enum of events dispatched by the Tabs component.
 * @enum {string}
 */
pricecheck.ui.tabs.Event = {
    CHANGE: 'tab-change',
    ACTION: 'tab-action'
};
